@@ -39,6 +39,7 @@
 #include "src/common/print_fields.h"
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
+#include "src/common/sluid.h"
 
 int print_fields_parsable_print = 0;
 int print_fields_have_header = 1;
@@ -62,7 +63,7 @@ extern void destroy_print_field(void *object)
 	}
 }
 
-extern void print_fields_header(List print_fields_list)
+extern void print_fields_header(list_t *print_fields_list)
 {
 	list_itr_t *itr = NULL;
 	print_field_t *field = NULL;
@@ -417,14 +418,40 @@ extern void print_fields_time_from_secs(print_field_t *field,
 	}
 }
 
+extern void print_fields_sluid(print_field_t *field, void *input, int last)
+{
+	int abs_len = abs(field->len);
+	sluid_t sluid = 0;
+	char *sluid_str = NULL;
+
+	if (input)
+		sluid = *(sluid_t *) input;
+
+	sluid_str = sluid2str(sluid);
+
+	if ((print_fields_parsable_print == PRINT_FIELDS_PARSABLE_NO_ENDING)
+	    && last)
+		printf("%s", sluid_str);
+	else if (print_fields_parsable_print && !fields_delimiter)
+		printf("%s|", sluid_str);
+	else if (print_fields_parsable_print && fields_delimiter)
+		printf("%s%s", sluid_str, fields_delimiter);
+	else if (field->len == abs_len)
+		printf("%*s ", abs_len, sluid_str);
+	else
+		printf("%-*s ", abs_len, sluid_str);
+
+	xfree(sluid_str);
+}
+
 extern void print_fields_char_list(print_field_t *field, void *input, int last)
 {
 	int abs_len = abs(field->len);
 	char *print_this = NULL;
-	List value = NULL;
+	list_t *value = NULL;
 
 	if (input)
-		value = *(List *) input;
+		value = *(list_t **) input;
 
 	if (!value || !list_count(value)) {
 		if (print_fields_parsable_print)
@@ -454,20 +481,29 @@ extern void print_fields_char_list(print_field_t *field, void *input, int last)
 	xfree(print_this);
 }
 
-static bool _is_wildcard(char *ptr)
+static bool _is_wildcard(char *ptr, job_std_pattern_t *job)
 {
 	switch (*ptr) {
 	case 'A': /* Array job ID */
 	case 'a': /* Array task ID */
+	case 'b': /* Array task ID modulo 10 */
 	case 'J': /* Jobid.stepid */
 	case 'j': /* Job ID */
+	case 'r': /* Restart count */
+	case 'S': /* SLUID */
+	case 's': /* Stepid of the running job */
+	case 'x': /* Job name */
+		return true;
+		break;
 	case 'N': /* Short hostname */
 	case 'n': /* Node id relative to current job */
-	case 's': /* Stepid of the running job */
 	case 't': /* Task id (rank) relative to current job */
 	case 'u': /* User name */
-	case 'x':
-		return true;
+		if (job->is_srun) { /* srun leaves these wildcards untouched */
+			job->io_per_task = true; /* set flag for srun to use */
+			return false;
+		} else
+			return true;
 		break;
 	default:
 		break;
@@ -480,28 +516,61 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 {
 	switch (*ptr) {
 	case 'A': /* Array job ID */
+		if (job->array_job_id) {
+			xstrfmtcatat(*expanded, pos, "%0*u", padding,
+				     job->array_job_id);
+		} else {
+			xstrfmtcatat(*expanded, pos, "%0*u", padding,
+				     job->step_id.job_id);
+		}
+		break;
 	case 'J': /* Jobid.stepid */
 	case 'j': /* Job ID */
-		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->jobid);
+		xstrfmtcatat(*expanded, pos, "%0*u", padding,
+			     job->step_id.job_id);
+		if ((*ptr == 'J') &&
+		    (job->step_id.step_id != SLURM_BATCH_SCRIPT))
+			xstrfmtcatat(*expanded, pos, ".%d",
+				     job->step_id.step_id);
 		break;
 	case 'a': /* Array task ID */
 		xstrfmtcatat(*expanded, pos, "%0*u", padding,
 			     job->array_task_id);
 		break;
+	case 'b': /* Array task ID modulo 10 */
+		xstrfmtcatat(*expanded, pos, "%0*u", padding,
+			     job->array_task_id % 10);
+		break;
 	case 'N': /* Short hostname */
 		xstrfmtcatat(*expanded, pos, "%s", job->first_step_node);
 		break;
+	case 'r':
+		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->restart_cnt);
+		break;
+	case 'S': /* SLUID */
+	{
+		char sluid[SLUID_STR_BYTES];
+		print_sluid(job->step_id.sluid, sluid, sizeof(sluid));
+		xstrfmtcatat(*expanded, pos, "%s", sluid);
+		break;
+	}
 	case 's': /* Stepid of the running job */
-		xstrfmtcatat(*expanded, pos, "%s", job->first_step_name);
+		if (job->step_id.step_id == SLURM_BATCH_SCRIPT)
+			xstrcatat(*expanded, pos, "batch");
+		else
+			xstrfmtcatat(*expanded, pos, "%0*u", padding,
+				     job->step_id.step_id);
 		break;
 	case 'n': /* Node id relative to current job */
+		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->nodeid);
+		break;
 	case 't': /* Task id (rank) relative to current job */
-		xstrfmtcatat(*expanded, pos, "0");
+		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->taskid);
 		break;
 	case 'u': /* User name */
 		xstrfmtcatat(*expanded, pos, "%s", job->user);
 		break;
-	case 'x':
+	case 'x': /* Job name */
 		xstrfmtcatat(*expanded, pos, "%s", job->jobname);
 		break;
 	default:
@@ -511,10 +580,6 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 
 /*
  * Special expansion function for stdin/stdout/stderr filename patterns.
- * Fields that can potientially map to a range of values will use the first in
- * that range (e.g %t is replaced by 0).
- *
- * The parser do not support steps and is only for batch jobs.
  *
  * \      If we found this symbol, don't replace anything.
  * %%     The character "%".
@@ -549,8 +614,13 @@ extern char *expand_stdio_fields(char *stdio_path, job_std_pattern_t *job)
 	if (!stdio_path || !*stdio_path || !job)
 		return NULL;
 
-	if (stdio_path[0] != '/')
-		xstrcatat(expanded, &pos, job->work_dir);
+	if (!job->is_srun && job->work_dir && (stdio_path[0] != '/')) {
+		size_t len = strlen(job->work_dir);
+		if (job->work_dir[len - 1] == '/')
+			xstrcatat(expanded, &pos, job->work_dir);
+		else
+			xstrfmtcatat(expanded, &pos, "%s/", job->work_dir);
+	}
 
 	/*
 	 * Special case, if we find a \ it means the file has not been
@@ -573,24 +643,48 @@ extern char *expand_stdio_fields(char *stdio_path, job_std_pattern_t *job)
 				xstrfmtcatat(expanded, &pos, "%c", *ptr);
 			break;
 		case STATE_EXPAND:
+			/* Double %% is escape, so print one %. */
+			if (*ptr == '%') {
+				xstrfmtcatat(expanded, &pos, "%c", *ptr);
+				curr_state = STATE_INIT;
+				break;
+			}
 			if (isdigit(*ptr)) {
+				char *tmp_ptr = ptr;
 				if ((padding = strtoul(ptr, &end, 10)) > 9) {
 					/* Remove % and double digit 10 */
 					ptr = end;
 					padding = 10;
-				} else
+				} else {
 					ptr++;
+				}
+				/*
+				 * weird behavior fix: we remove all the digits
+				 * except the last one to match with the current
+				 * fname creation.
+				 */
+				if (!_is_wildcard(ptr, job)) {
+					ptr = tmp_ptr;
+					/* seek until the last digit */
+					while (isdigit(*(ptr + 1))) {
+						ptr++;
+					}
+					xstrfmtcatat(expanded, &pos,
+						     "%c", *ptr);
+					padding = 0;
+					curr_state = STATE_INIT;
+					break;
+				}
 			}
-			if (!_is_wildcard(ptr)) {
-				padding = 0;
-				xstrfmtcatat(expanded, &pos, "%c", *ptr);
+			if (!_is_wildcard(ptr, job)) {
+				/* If not a wildcard print also the %. */
+				xstrfmtcatat(expanded, &pos, "%%%c", *ptr);
 			} else {
 				_expand_wildcard(&expanded, &pos, ptr, padding,
 						 job);
+				padding = 0;
 			}
-			/* If we find another %, don't leave this state yet. */
-			if (*ptr != '%')
-				curr_state = STATE_INIT;
+			curr_state = STATE_INIT;
 			break;
 		default:
 			break;
