@@ -280,7 +280,7 @@ static void _job_post_update(ctxt_t *ctxt, slurm_selected_step_t *job_id)
 	}
 
 	if (job_id->step_id.job_id != NO_VAL)
-		job->job_id = job_id->step_id.job_id;
+		job->step_id.job_id = job_id->step_id.job_id;
 
 	if (job_id->het_job_offset != NO_VAL)
 		job->het_job_offset = job_id->het_job_offset;
@@ -369,8 +369,9 @@ static void _job_post_submit(ctxt_t *ctxt, job_desc_msg_t *job, char *script)
 		};
 
 		debug3("%s:[%s] job submitted -> job_id:%d step_id:%d rc:%d message:%s",
-		       __func__, ctxt->id, resp->job_id, resp->step_id,
-		       resp->error_code, resp->job_submit_user_msg);
+		       __func__, ctxt->id, resp->step_id.job_id,
+		       resp->step_id.step_id, resp->error_code,
+		       resp->job_submit_user_msg);
 
 		if (resp->error_code)
 			resp_warn(ctxt, "slurm_submit_batch_job()",
@@ -429,8 +430,9 @@ static void _job_post_het_submit(ctxt_t *ctxt, list_t *jobs, char *script)
 		};
 
 		debug3("%s:[%s] HetJob submitted -> job_id:%d step_id:%d rc:%d message:%s",
-		       __func__, ctxt->id, resp->job_id, resp->step_id,
-		       resp->error_code, resp->job_submit_user_msg);
+		       __func__, ctxt->id, resp->step_id.job_id,
+		       resp->step_id.step_id, resp->error_code,
+		       resp->job_submit_user_msg);
 
 		if (resp->error_code)
 			resp_warn(ctxt, "slurm_submit_batch_het_job()",
@@ -474,8 +476,8 @@ static void _job_post(ctxt_t *ctxt)
 		goto cleanup;
 
 	if (!req.jobs && (!req.script || !req.script[0]) &&
-	    (!req.job || !req.job->script) &&
-	    !(req.job->bitflags & EXTERNAL_JOB)) {
+	    (!req.job ||
+	     (!req.job->script && !(req.job->bitflags & EXTERNAL_JOB)))) {
 		resp_error(ctxt, ESLURM_REST_INVALID_QUERY, __func__,
 			   "Populated \"script\" field is required for job submission");
 		goto cleanup;
@@ -601,20 +603,21 @@ static int _foreach_alloc_job_resp(void *x, void *arg)
 
 	xassert(args->magic == FOREACH_ALLOC_JOB_ARGS_MAGIC);
 
-	xassert(!oas_resp->job_id || (oas_resp->job_id == resp->job_id) ||
-		(oas_resp->job_id == (resp->job_id - args->component)));
-	oas_resp->job_id = resp->job_id;
+	xassert(!oas_resp->job_id ||
+		(oas_resp->job_id == resp->step_id.job_id) ||
+		(oas_resp->job_id == (resp->step_id.job_id - args->component)));
+	oas_resp->job_id = resp->step_id.job_id;
 
 	if (!oas_resp->job_submit_user_msg)
 		oas_resp->job_submit_user_msg = resp->job_submit_user_msg;
 
 	if (args->component == NO_VAL) {
 		debug3("%s:[%s] Job submitted -> JobId=%d rc:%d message:%s",
-		       __func__, ctxt->id, resp->job_id,
+		       __func__, ctxt->id, resp->step_id.job_id,
 		       resp->error_code, resp->job_submit_user_msg);
 	} else {
 		debug3("%s:[%s] HetJob submitted -> JobId=%d+%d rc:%d message:%s",
-		       __func__, ctxt->id, resp->job_id, args->component,
+		       __func__, ctxt->id, resp->step_id.job_id, args->component,
 		       resp->error_code, resp->job_submit_user_msg);
 		args->component++;
 	}
