@@ -61,8 +61,10 @@
 #include "src/common/read_config.h"
 #include "src/common/ref.h"
 #include "src/common/run_in_daemon.h"
+#include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/uid.h"
+#include "src/common/util-net.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
@@ -73,9 +75,11 @@
 #include "src/interfaces/cred.h"
 #include "src/interfaces/data_parser.h"
 #include "src/interfaces/hash.h"
+#include "src/interfaces/http_parser.h"
 #include "src/interfaces/select.h"
 #include "src/interfaces/serializer.h"
 #include "src/interfaces/tls.h"
+#include "src/interfaces/url_parser.h"
 
 #include "src/slurmrestd/http.h"
 #include "src/slurmrestd/openapi.h"
@@ -87,6 +91,8 @@
 #define OPT_LONG_GEN_OAS 0x102
 
 #define SLURM_CONF_DISABLED "/dev/null"
+#define DEFAULT_OPENAPI_PLUGINS_SLURMDBD "slurmctld,slurmdbd,util"
+#define DEFAULT_OPENAPI_PLUGINS "slurmctld,util"
 
 #if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
 #define unshare(_) (false)
@@ -97,16 +103,15 @@ decl_static_data(usage_txt);
 uint32_t slurm_daemon = IS_SLURMRESTD;
 
 typedef struct {
-	bool stdin_tty; /* running with a TTY for stdin */
 	bool stdin_socket; /* running with a socket for stdin */
 	bool stderr_tty; /* running with a TTY for stderr */
-	bool stdout_tty; /* running with a TTY for stdout */
-	bool stdout_socket; /* running with a socket for stdout */
 	bool listen; /* running in listening daemon mode aka not INET mode */
 } run_mode_t;
 
 /* Debug level to use */
 static int debug_level = 0;
+static int debug_level_syslog = 0;
+static int debug_level_stderr = 0;
 static int debug_increase = 0;
 /* detected run mode */
 static run_mode_t run_mode = { 0 };
@@ -137,9 +142,6 @@ static bool check_user = true;
 static bool become_user = false;
 static http_status_code_t *response_status_codes = NULL;
 
-extern parsed_host_port_t *parse_host_port(const char *str);
-extern void free_parse_host_port(parsed_host_port_t *parsed);
-
 static void _plugrack_foreach_list(const char *full_type, const char *fq_path,
 				   const plugin_handle_t id, void *arg)
 {
@@ -149,9 +151,20 @@ static void _plugrack_foreach_list(const char *full_type, const char *fq_path,
 /* SIGPIPE handler - mostly a no-op */
 static void _sigpipe_handler(conmgr_callback_args_t conmgr_args, void *arg)
 {
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED)
+		return;
+
 	debug5("%s: received SIGPIPE", __func__);
 }
 
+static void _on_sigprof(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED)
+		return;
+
+	conmgr_log_diagnostics();
+}
+
 static void _set_max_connections(const char *buffer)
 {
 	max_connections = slurm_atoul(buffer);
@@ -166,11 +179,31 @@ static void _parse_env(void)
 {
 	char *buffer = NULL;
 
-	if ((buffer = getenv("SLURMRESTD_DEBUG")) != NULL) {
+	if ((buffer = getenv("SLURMRESTD_DEBUG")) ||
+	    (buffer = getenv("SLURM_DEBUG"))) {
 		debug_level = log_string2num(buffer);
 
 		if ((debug_level < 0) || (debug_level == NO_VAL16))
-			fatal("Invalid env SLURMRESTD_DEBUG: %s", buffer);
+			fatal("Invalid env SLURMRESTD_DEBUG or SLURM_DEBUG: %s",
+			      buffer);
+	}
+
+	if ((buffer = getenv("SLURMRESTD_DEBUG_SYSLOG")) != NULL) {
+		debug_level_syslog = log_string2num(buffer);
+
+		if ((debug_level_syslog < 0) ||
+		    (debug_level_syslog == NO_VAL16))
+			fatal("Invalid env SLURMRESTD_DEBUG_SYSLOG: %s",
+			      buffer);
+	}
+
+	if ((buffer = getenv("SLURMRESTD_DEBUG_STDERR")) != NULL) {
+		debug_level_stderr = log_string2num(buffer);
+
+		if ((debug_level_stderr < 0) ||
+		    (debug_level_stderr == NO_VAL16))
+			fatal("Invalid env SLURMRESTD_DEBUG_STDERR: %s",
+			      buffer);
 	}
 
 	if ((buffer = getenv("SLURMRESTD_LISTEN")) != NULL) {
@@ -240,7 +273,7 @@ static void _parse_env(void)
 		while (token) {
 			http_status_code_t code = get_http_status_code(token);
 
-			if (code == HTTP_STATUS_NONE)
+			if (code == HTTP_STATUS_CODE_INVALID)
 				fatal("Unable to parse %s as HTTP status code",
 				      token);
 
@@ -255,7 +288,7 @@ static void _parse_env(void)
 		xfree(toklist);
 
 		if (response_status_codes)
-			response_status_codes[count] = HTTP_STATUS_NONE;
+			response_status_codes[count] = HTTP_STATUS_CODE_INVALID;
 	}
 }
 
@@ -268,9 +301,6 @@ static void _examine_stdin(void)
 
 	if ((status.st_mode & S_IFMT) == S_IFSOCK)
 		run_mode.stdin_socket = true;
-
-	if (isatty(STDIN_FILENO))
-		run_mode.stdin_tty = true;
 }
 
 static void _examine_stderr(void)
@@ -284,18 +314,9 @@ static void _examine_stderr(void)
 		run_mode.stderr_tty = true;
 }
 
-static void _examine_stdout(void)
+static int _add_log_level(int base, int increase)
 {
-	struct stat status = { 0 };
-
-	if (fstat(STDOUT_FILENO, &status))
-		fatal("unable to stat STDOUT: %m");
-
-	if ((status.st_mode & S_IFMT) == S_IFSOCK)
-		run_mode.stdout_socket = true;
-
-	if (isatty(STDOUT_FILENO))
-		run_mode.stdout_tty = true;
+	return MIN((base + increase), (LOG_LEVEL_END - 1));
 }
 
 static void _setup_logging(int argc, char **argv)
@@ -303,31 +324,45 @@ static void _setup_logging(int argc, char **argv)
 	/* Default to logging as a daemon */
 	log_options_t logopt = LOG_OPTS_INITIALIZER;
 	log_facility_t fac = SYSLOG_FACILITY_DAEMON;
+	bool interactive_mode = false;
+
+	if (debug_level_syslog)
+		logopt.syslog_level =
+			_add_log_level(debug_level_syslog, debug_increase);
+	if (debug_level_stderr)
+		logopt.stderr_level =
+			_add_log_level(debug_level_stderr, debug_increase);
 
 	/*
-	 * Set debug level as requested.
-	 * debug_level is set to the value of SLURMRESTD_DEBUG.
-	 * SLURMRESTD_DEBUG sets the debug level if -v's are not given.
-	 * debug_increase is the command line option -v, which applies on top
-	 * of the default log level (info).
+	 * Only activate interactive mode to log to TTY if syslog and
+	 * stderr were not explicitly set
 	 */
-	if (debug_increase)
-		debug_level = MIN((LOG_LEVEL_INFO + debug_increase),
-				  (LOG_LEVEL_END - 1));
-	else if (!debug_level)
-		debug_level = LOG_LEVEL_INFO;
+	if (!debug_level_syslog && !debug_level_stderr) {
+		if (!debug_level)
+			debug_level =
+				_add_log_level(LOG_LEVEL_INFO, debug_increase);
+		else
+			debug_level =
+				_add_log_level(debug_level, debug_increase);
 
-	logopt.syslog_level = debug_level;
+		if (run_mode.stderr_tty) {
+			logopt = (log_options_t) LOG_OPTS_STDERR_ONLY;
+			fac = SYSLOG_FACILITY_USER;
+			logopt.stderr_level = debug_level;
 
-	if (run_mode.stderr_tty) {
-		/* Log to stderr if it is a tty */
-		logopt = (log_options_t) LOG_OPTS_STDERR_ONLY;
-		fac = SYSLOG_FACILITY_USER;
-		logopt.stderr_level = debug_level;
+			interactive_mode = true;
+		} else {
+			/* Default to only logging to syslog */
+			logopt.syslog_level =
+				_add_log_level(debug_level, debug_increase);
+		}
 	}
 
 	if (log_init(xbasename(argv[0]), logopt, fac, NULL))
 		fatal("Unable to setup logging: %m");
+
+	if (interactive_mode)
+		debug("Interactive mode activated (TTY detected on STDERR)");
 }
 
 /*
@@ -643,13 +678,6 @@ static void _check_user(void)
 	}
 }
 
-/* simple wrapper to hand over operations router in http context */
-static void *_setup_http_context(conmgr_fd_t *con, void *arg)
-{
-	xassert(operations_router == arg);
-	return setup_http_context(con, operations_router);
-}
-
 static void _auth_plugrack_foreach(const char *full_type, const char *fq_path,
 				   const plugin_handle_t id, void *arg)
 {
@@ -668,36 +696,40 @@ static void _auth_plugrack_foreach(const char *full_type, const char *fq_path,
 
 static void _on_signal_interrupt(conmgr_callback_args_t conmgr_args, void *arg)
 {
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED)
+		return;
+
 	info("%s: caught SIGINT. Shutting down.", __func__);
 	conmgr_request_shutdown();
 }
 
-
-static void _inet_on_finish(conmgr_fd_t *con, void *ctxt)
+static void _load_oas_specs(void)
 {
-	on_http_connection_finish(con, ctxt);
-	conmgr_request_shutdown();
+	if (oas_specs && !xstrcasecmp(oas_specs, "list")) {
+		fprintf(stderr, "Possible OpenAPI plugins:\n");
+		(void) init_openapi(oas_specs, _plugrack_foreach_list, NULL,
+				    NULL);
+		exit(0);
+	}
+
+	if (!oas_specs) {
+		if (slurm_with_slurmdbd())
+			oas_specs = xstrdup(DEFAULT_OPENAPI_PLUGINS_SLURMDBD);
+		else
+			oas_specs = xstrdup(DEFAULT_OPENAPI_PLUGINS);
+	}
+
+	if (init_openapi(oas_specs, NULL, parsers, response_status_codes))
+		fatal("Unable to initialize OpenAPI structures");
+
+	xfree(oas_specs);
 }
 
 int main(int argc, char **argv)
 {
 	int rc = SLURM_SUCCESS, parse_rc = SLURM_SUCCESS;
 	socket_listen = list_create(xfree_ptr);
-	static const conmgr_events_t conmgr_events = {
-		.on_data = parse_http,
-		.on_connection = _setup_http_context,
-		.on_finish = on_http_connection_finish,
-		.on_fingerprint = on_fingerprint_tls,
-	};
-	static const conmgr_events_t inet_events = {
-		.on_data = parse_http,
-		.on_connection = _setup_http_context,
-		.on_finish = _inet_on_finish,
-	};
-	static const conmgr_callbacks_t callbacks = {
-		.parse = parse_host_port,
-		.free_parse = free_parse_host_port,
-	};
+	conmgr_con_flags_t flags = CON_FLAG_NONE;
 
 	_parse_env();
 	_parse_commandline(argc, argv);
@@ -710,7 +742,6 @@ int main(int argc, char **argv)
 
 	_examine_stdin();
 	_examine_stderr();
-	_examine_stdout();
 	_setup_logging(argc, argv);
 
 	run_mode.listen = !list_is_empty(socket_listen);
@@ -724,16 +755,26 @@ int main(int argc, char **argv)
 		serializer_required(MIME_TYPE_YAML);
 	serializer_required(MIME_TYPE_URL_ENCODED);
 
+	if ((rc = http_parser_g_init()))
+		fatal("Unable to load http_parser plugin: %s",
+		      slurm_strerror(rc));
+	if ((rc = url_parser_g_init()))
+		fatal("Unable to load url_parser plugin: %s",
+		      slurm_strerror(rc));
+
 	/* This checks if slurmrestd is running in inetd mode */
-	conmgr_init((run_mode.listen ? thread_count : CONMGR_THREAD_COUNT_MIN),
-		    max_connections, callbacks);
+	conmgr_init(thread_count,
+		    (run_mode.listen ? 0 : CONMGR_THREAD_COUNT_MIN),
+		    max_connections);
 
 	/*
 	 * Attempt to load TLS plugin and then attempt to load the certificate
 	 * or give user warning TLS will not be supported
 	 */
 	if (!tls_g_init() && tls_available() &&
-	    (rc = tls_g_load_own_cert(NULL, 0, NULL, 0))) {
+	    !tls_g_load_own_cert(NULL, 0, NULL, 0)) {
+		flags |= CON_FLAG_TLS_FINGERPRINT;
+	} else {
 		debug("Disabling TLS support due to failure loading TLS certificate");
 
 		if ((rc = tls_g_fini()))
@@ -743,6 +784,7 @@ int main(int argc, char **argv)
 
 	conmgr_add_work_signal(SIGINT, _on_signal_interrupt, NULL);
 	conmgr_add_work_signal(SIGPIPE, _sigpipe_handler, NULL);
+	conmgr_add_work_signal(SIGPROF, _on_sigprof, NULL);
 
 	auth_rack = plugrack_create("rest_auth");
 	plugrack_read_dir(auth_rack, slurm_conf.plugindir);
@@ -802,15 +844,7 @@ int main(int argc, char **argv)
 	if (init_operations(parsers))
 		fatal("Unable to initialize operations structures");
 
-	if (oas_specs && !xstrcasecmp(oas_specs, "list")) {
-		fprintf(stderr, "Possible OpenAPI plugins:\n");
-		init_openapi(oas_specs, _plugrack_foreach_list, NULL, NULL);
-		exit(0);
-	} else if (init_openapi(oas_specs, NULL, parsers,
-				response_status_codes))
-		fatal("Unable to initialize OpenAPI structures");
-
-	xfree(oas_specs);
+	_load_oas_specs();
 
 	/* Sanity check modes */
 	if (run_mode.stdin_socket) {
@@ -824,14 +858,12 @@ int main(int argc, char **argv)
 		xfree(out);
 	}
 
-	if (run_mode.stdin_tty)
-		debug("Interactive mode activated (TTY detected on STDIN)");
-
 	if (!run_mode.listen) {
+		inetd_mode = true;
 		if ((rc = conmgr_process_fd(CON_TYPE_RAW, STDIN_FILENO,
-					    STDOUT_FILENO, &inet_events,
-					    CON_FLAG_NONE, NULL, 0,
-					    NULL, operations_router)))
+					    STDOUT_FILENO, http_events_get(),
+					    flags, NULL, 0, NULL,
+					    operations_router)))
 			fatal("%s: unable to process stdin: %s",
 			      __func__, slurm_strerror(rc));
 
@@ -840,8 +872,9 @@ int main(int argc, char **argv)
 	} else if (run_mode.listen) {
 		mode_t mask = umask(0);
 
-		if (conmgr_create_listen_sockets(CON_TYPE_RAW, CON_FLAG_NONE,
-						 socket_listen, &conmgr_events,
+		if (conmgr_create_listen_sockets(CON_TYPE_RAW, flags,
+						 socket_listen,
+						 http_events_get(),
 						 operations_router))
 			fatal("Unable to create sockets");
 
@@ -879,12 +912,15 @@ int main(int argc, char **argv)
 	auth_rack = NULL;
 
 	xfree(auth_plugin_handles);
+	http_parser_g_fini();
+	url_parser_g_fini();
 	acct_storage_g_fini();
 	slurm_fini();
 	hash_g_fini();
 	conn_g_fini();
 	cred_g_fini();
 	auth_g_fini();
+	getnameinfo_cache_purge();
 	log_fini();
 
 	/* send parsing RC if there were no higher level errors */
